@@ -1,13 +1,14 @@
 from __future__ import absolute_import
-from abc import ABCMeta, abstractmethod
+import abc
 import collections
 import copy
 import errno
 import fnmatch
 import inspect
 import os
+import six
+import yaml
 import pickle
-from future.utils import with_metaclass
 
 from . import loaders
 from . import futures
@@ -56,9 +57,28 @@ class Bundle(dict):
         return Savable.load(self, load_context)
 
 
-class Persister(with_metaclass(ABCMeta, object)):
+_BUNDLE_TAG = u'!plumpy:Bundle'
 
-    @abstractmethod
+
+def _bundle_representer(dumper, node):
+    return dumper.represent_mapping(_BUNDLE_TAG, node)
+
+
+def _bundle_constructor(loader, data):
+    result = Bundle.__new__(Bundle)
+    yield result
+    mapping = loader.construct_mapping(data)
+    result.update(mapping)
+
+
+yaml.add_representer(Bundle, _bundle_representer)
+yaml.add_constructor(_BUNDLE_TAG, _bundle_constructor)
+
+
+@six.add_metaclass(abc.ABCMeta)
+class Persister(object):
+
+    @abc.abstractmethod
     def save_checkpoint(self, process, tag=None):
         """
         Persist a Process instance
@@ -70,7 +90,7 @@ class Persister(with_metaclass(ABCMeta, object)):
         """
         pass
 
-    @abstractmethod
+    @abc.abstractmethod
     def load_checkpoint(self, pid, tag=None):
         """
         Load a process from a persisted checkpoint by its process id
@@ -84,7 +104,7 @@ class Persister(with_metaclass(ABCMeta, object)):
         """
         pass
 
-    @abstractmethod
+    @abc.abstractmethod
     def get_checkpoints(self):
         """
         Return a list of all the current persisted process checkpoints
@@ -94,7 +114,7 @@ class Persister(with_metaclass(ABCMeta, object)):
         """
         pass
 
-    @abstractmethod
+    @abc.abstractmethod
     def get_process_checkpoints(self, pid):
         """
         Return a list of all the current persisted process checkpoints for the
@@ -106,7 +126,7 @@ class Persister(with_metaclass(ABCMeta, object)):
         """
         pass
 
-    @abstractmethod
+    @abc.abstractmethod
     def delete_checkpoint(self, pid, tag=None):
         """
         Delete a persisted process checkpoint. No error will be raised if
@@ -118,7 +138,7 @@ class Persister(with_metaclass(ABCMeta, object)):
         """
         pass
 
-    @abstractmethod
+    @abc.abstractmethod
     def delete_process_checkpoints(self, pid):
         """
         Delete all persisted checkpoints related to the given process id
@@ -241,7 +261,7 @@ class PicklePersister(Persister):
         checkpoints = []
         file_pattern = '*.{}'.format(_PICKLE_SUFFIX)
 
-        for subdir, dirs, files in os.walk(self._pickle_directory):
+        for _subdir, dirs, files in os.walk(self._pickle_directory):
             for filename in fnmatch.filter(files, file_pattern):
                 filepath = os.path.join(self._pickle_directory, filename)
                 persisted_pickle = PicklePersister.load_pickle(filepath)
@@ -286,7 +306,8 @@ class PicklePersister(Persister):
             self.delete_checkpoint(checkpoint.pid, checkpoint.tag)
 
 
-class InMemoryPersister(with_metaclass(ABCMeta, object)):
+@six.add_metaclass(abc.ABCMeta)
+class InMemoryPersister(object):
     """ Mainly to be used in testing/debugging """
 
     def __init__(self, loader=None):
